@@ -8,18 +8,21 @@ use pea2pea::{
     Config, Node, Pea2Pea,
 };
 
-use std::time::Duration;
+use std::time::{Duration, Instant};
 
 #[tokio::test(flavor = "multi_thread")]
 async fn fuzzing() {
     // tracing_subscriber::fmt::init();
 
+    let start = Instant::now();
+
     const MAX_MSG_SIZE: usize = 4096;
-    const NUM_MESSAGES: usize = 1_000;
+    const ITERATIONS: usize = 1000;
+    const MSGS_PER_ITERATION: usize = 4;
 
     let config = Config {
         name: Some("receiver".into()),
-        read_buffer_size: MAX_MSG_SIZE,
+        read_buffer_size: MAX_MSG_SIZE * MSGS_PER_ITERATION,
         ..Default::default()
     };
     let receiver = common::MessagingNode(Node::new(Some(config)).await.unwrap());
@@ -43,21 +46,25 @@ async fn fuzzing() {
     let mut expected_msg_size: u64 = 0;
     let mut processed_sizes = CircularQueue::with_capacity(256);
 
-    for _ in 0..NUM_MESSAGES {
-        let random_len: usize = rng.gen_range(1..=MAX_MSG_SIZE - 4); // account for the length prefix
-        let random_payload: Vec<u8> = (&mut rng).sample_iter(Standard).take(random_len).collect();
+    for _ in 0..ITERATIONS {
+        for _ in 0..MSGS_PER_ITERATION {
+            let random_len: usize = rng.gen_range(1..=MAX_MSG_SIZE - 4); // account for the length prefix
+            let random_payload: Vec<u8> =
+                (&mut rng).sample_iter(Standard).take(random_len).collect();
+
+            sender
+                .send_direct_message(receiver_addr, random_payload.into())
+                .unwrap();
 
-        // skip full channel errors
-        if sender
-            .send_direct_message(receiver_addr, random_payload.into())
-            .is_ok()
-        {
             processed_sizes.push(random_len);
             expected_msg_count += 1;
             expected_msg_size += 4 + random_len as u64;
         }
 
-        sleep(Duration::from_millis(5)).await;
+        wait_until!(
+            1,
+            receiver.node().stats().received() == (expected_msg_count, expected_msg_size)
+        );
 
         if receiver.node().num_connected() == 0 {
             let last_processed = processed_sizes.asc_iter().collect::<Vec<_>>();
@@ -65,9 +72,14 @@ async fn fuzzing() {
         }
     }
 
-    wait_until!(
-        1,
-        receiver.node().stats().received() == (expected_msg_count, expected_msg_size)
+    let elapsed_secs = start.elapsed().as_millis() as f64 / 1000.0;
+    let bytes_per_s = expected_msg_size as f64 / elapsed_secs;
+
+    println!(
+        "fuzzing complete; fuzzed {} in {}s ({}/s)",
+        common::display_bytes(expected_msg_size as f64),
+        elapsed_secs,
+        common::display_bytes(bytes_per_s),
     );
 }
 
